/*
 * Copyright (c) 2007 Kathryn Huxtable
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: RequestWrapper.java 1072 2007-04-08 04:26:07Z huxtable $
 */
package org.kathrynhuxtable.middleware.shibshim.filter;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
 * <p>
 * A Servlet Filter wraps the Request with this type of subclass that intercepts
 * methods it wants to override. The wrapper class is then passed on through the
 * chain to the next Filter or Servlet.
 * </p>
 * 
 * <p>
 * This class overrides the header methods to slip the Shibboleth attributes in
 * as HTTP headers.
 * </p>
 * 
 * <p>
 * At attribute beginning with the prefix but not found in the
 * UserAttributesImpl will not be returned. This is to prevent spoofing of
 * attributes.
 * </p>
 */
public class RequestWrapper extends HttpServletRequestWrapper {

    /**
     * The original request object.
     */
    private HttpServletRequest request;

    /**
     * The attributes object, used to return headers and such.
     */
    private UserAttributesImpl attributes;

    /**
     * The name of the attribute to be return from getRemoteUser.
     */
    private String             remoteUserAttribute;

    /**
     * Construct an HTTPServletRequestWrapper to return the Shibboleth
     * attributes as HTTP headers.
     * 
     * @param request
     *            The original request object
     * @param attributes
     *            The Argus attributes object
     * @param remoteUserAttribute
     *            The name of the attribute to return from getRemoteUser
     */
    public RequestWrapper(HttpServletRequest request, UserAttributesImpl attributes, String remoteUserAttribute) {
        super(request);
        this.request = request;
        this.attributes = attributes;
        this.remoteUserAttribute = remoteUserAttribute;
    }

    /**
     * Return HTTP header, substituting the attributes in our attributes object
     * for any attributes that begin with the prefix string.
     * 
     * @param name
     *            the attribute name
     * @return the first value of the header.
     */
    public String getHeader(String name) {
        name = name.toLowerCase();
        if (attributes != null && name != null && attributes.isMappedAttribute(name)) {
            return attributes.getHeader(name);
        }
        return request.getHeader(name);
    }

    /**
     * Return the HTTP header names, including the Shibboleth attributes.
     * 
     * @return all of the header names.
     */
    public Enumeration getHeaderNames() {
        Enumeration names = request.getHeaderNames();

        if (attributes != null) {
            Vector v = new Vector(Collections.list(names));
            Enumeration keys = attributes.getHeaderNames();
            while (keys.hasMoreElements()) {
                String key = (String) keys.nextElement();
                if (attributes.isMappedAttribute(key) && !v.contains(key)) {
                    v.add(key);
                }
            }
            names = v.elements();
        }

        return names;
    }

    /**
     * Return the values for a header, allowing multiple values and substituting
     * the attributes in our attributes object for any attributes that begin
     * with the prefix string.
     * 
     * @param name
     *            the name of the header.
     * @return all the values for the header name.
     */
    public Enumeration getHeaders(String name) {
        name = name.toLowerCase();
        if (attributes != null && name != null && attributes.isMappedAttribute(name)) {
            return attributes.getHeaders(name);
        }
        return request.getHeaders(name);
    }

    /**
     * Return the Shibboleth attribute for the remote user, unless the name is
     * not defined, in which case, return the original remote user.
     * 
     * <p>
     * Note that if the attribute is specified but not defined, then null is
     * returned.
     * </p>
     * 
     * @return the remote username. This will normally be the
     *         eduPersonPrincipalName.
     */
    public String getRemoteUser() {
        if (remoteUserAttribute != null) {
            return getHeader(remoteUserAttribute);
        }
        return request.getRemoteUser();
    }
}
